package cn.iocoder.yudao.module.reward.service.redeem;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.map.MapUtil;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.dict.core.dto.DictDataRespDTO;
import cn.iocoder.yudao.framework.dict.core.util.DictFrameworkUtils;
import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.ecw.api.currency.CurrencyApi;
import cn.iocoder.yudao.module.ecw.api.currency.dto.CurrencyRespDTO;
import cn.iocoder.yudao.module.ecw.api.express.ExpressApi;
import cn.iocoder.yudao.module.ecw.api.express.dto.ExpressRespDTO;
import cn.iocoder.yudao.module.member.api.score.MemberUserScoreApi;
import cn.iocoder.yudao.module.member.api.score.dto.MemberUserScoreOperateReqDTO;
import cn.iocoder.yudao.module.member.api.score.dto.ReleationScoreExpireInfoDTO;
import cn.iocoder.yudao.module.member.enums.PlatformTypeEnum;
import cn.iocoder.yudao.module.member.enums.ScoreSourceTypeEnum;
import cn.iocoder.yudao.module.reward.controller.app.redeem.vo.AppRewardRedeemListRespVO;
import cn.iocoder.yudao.module.reward.dal.dataobject.redeem.RewardRedeemDO;
import cn.iocoder.yudao.module.reward.dal.dataobject.reward.RewardDO;
import cn.iocoder.yudao.module.reward.dal.mysql.redeem.RewardRedeemMapper;
import cn.iocoder.yudao.module.reward.dto.RewardRedeemVerifyDTO;
import cn.iocoder.yudao.module.reward.enums.ErrorCodeConstants;
import cn.iocoder.yudao.module.reward.enums.RewardPickMethedEnum;
import cn.iocoder.yudao.module.reward.enums.RewardRedeemStatusEnum;
import cn.iocoder.yudao.module.reward.service.reward.RewardService;
import cn.iocoder.yudao.module.reward.vo.reward.*;
import cn.iocoder.yudao.module.system.api.file.FileMakeApi;
import cn.iocoder.yudao.module.system.api.file.dto.FileMakeReqDTO;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

import static cn.hutool.core.date.DatePattern.NORM_DATETIME_PATTERN;
import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.system.enums.download.DownloadTypeEnum.REWARD_REDEEM_RECORD;

/**
 * 礼品兑换 Service
 *
 * @author 系统管理员
 */
@Service
@Slf4j
public class RewardRedeemServiceImpl extends AbstractService<RewardRedeemMapper, RewardRedeemDO> implements RewardRedeemService {

    private static final Integer MAX_IMPORT_RECORD_COUNT = 500;

    private final RewardRedeemMapper rewardRedeemMapper;
    private final FileMakeApi fileMakeApi;
    private final ExpressApi expressApi;
    private final CurrencyApi currencyApi;
    private final MemberUserScoreApi memberUserScoreApi;
    private final RewardService rewardService;

    public RewardRedeemServiceImpl(RewardRedeemMapper rewardRedeemMapper, FileMakeApi fileMakeApi,
                                   ExpressApi expressApi, CurrencyApi currencyApi, MemberUserScoreApi memberUserScoreApi, RewardService rewardService) {
        this.rewardRedeemMapper = rewardRedeemMapper;
        this.fileMakeApi = fileMakeApi;
        this.expressApi = expressApi;
        this.currencyApi = currencyApi;
        this.memberUserScoreApi = memberUserScoreApi;
        this.rewardService = rewardService;
    }

    @Override
    public PageResult<RewardRedeemPageRespVO> pageInfo(RewardRedeemPageReqVO reqVO) {
        int start = (reqVO.getPageNo() - 1) * reqVO.getPageSize();
        int size = reqVO.getPageSize();
        List<RewardRedeemPageRespVO> result = rewardRedeemMapper.pageInfo(start, size, reqVO);
        result.forEach(respVO -> {
            if (respVO.getEntrance() != PlatformTypeEnum.BACKEND.getValue()) {
                respVO.setCreatorName(respVO.getMemberNameZh());
                if (respVO.getUpdateTime().equals(respVO.getCreateTime())) {
                    respVO.setUpdaterName(respVO.getMemberNameZh());
                }
            }
        });
        int total = rewardRedeemMapper.pageCount(reqVO);
        return new PageResult<>(result, total, reqVO.getPageSize(), reqVO.getPageNo(), (total + reqVO.getPageSize() - 1) / reqVO.getPageSize());
    }

    @Override
    public RewardRedeemPageRespVO detail(Long id) {
        if (id == null) {
            return null;
        }
        RewardRedeemPageRespVO detail = rewardRedeemMapper.detail(id);
        if (detail != null && detail.getEntrance() != PlatformTypeEnum.BACKEND.getValue()) {
            detail.setCreatorName(detail.getMemberNameZh());
            // 刚创建时更新人显示为兑换人
            if (detail.getUpdateTime().equals(detail.getCreateTime())) {
                detail.setUpdaterName(detail.getMemberNameZh());
            }
        }
        return detail;
    }

    @Override
    public Boolean updateRecord(RewardRedeemUpdateReqVO request) {
        Long rewardId = updateValidateParam(request);
        RewardDO rewardDO = rewardService.getById(rewardId);
        if (rewardDO == null) {
            throw exception(ErrorCodeConstants.REWARD_NOT_EXISTS);
        }
        if (!Arrays.asList(rewardDO.getPickMethod().split(",")).contains(request.getRedeemType().toString())) {
            throw exception(ErrorCodeConstants.REWARD_PICK_METHOD_NOT_ALLOW);
        }
        LambdaUpdateWrapper<RewardRedeemDO> wrapper = Wrappers.lambdaUpdate();
        wrapper.eq(RewardRedeemDO::getId, request.getId());
        wrapper.set(RewardRedeemDO::getRedeemType, request.getRedeemType());
        wrapper.set(RewardRedeemDO::getExpenses, request.getExpenses());
        wrapper.set(RewardRedeemDO::getCurrency, request.getCurrency());
        wrapper.set(RewardRedeemDO::getRecipientName, request.getRecipientName());
        wrapper.set(RewardRedeemDO::getRecipientPhoneNum, request.getRecipientPhoneNum());
        wrapper.set(RewardRedeemDO::getRedeemer, request.getRedeemer());
        wrapper.set(RewardRedeemDO::getRedemptionTime, request.getRedemptionTime());
        wrapper.set(RewardRedeemDO::getExpressSender, request.getExpressSender());
        wrapper.set(RewardRedeemDO::getAnnex, request.getAnnex());
        wrapper.set(RewardRedeemDO::getRemark, request.getRemark());
        wrapper.set(RewardRedeemDO::getUpdater, SecurityFrameworkUtils.getLoginUserId());
        wrapper.set(RewardRedeemDO::getUpdateTime, new Date());
        if (request.getRedeemType() == RewardPickMethedEnum.SELF_PICKUP.getValue()) {
            wrapper.set(RewardRedeemDO::getRecipientAddress, "");
            wrapper.set(RewardRedeemDO::getCourierCompany, null);
            wrapper.set(RewardRedeemDO::getExpressNo, null);
            wrapper.set(RewardRedeemDO::getExpressDate, null);
        } else {
            wrapper.set(StringUtils.isNotBlank(request.getRecipientAddress()), RewardRedeemDO::getRecipientAddress, request.getRecipientAddress());
            wrapper.set(RewardRedeemDO::getCourierCompany, request.getCourierCompany());
            wrapper.set(RewardRedeemDO::getExpressNo, request.getExpressNo());
            wrapper.set(RewardRedeemDO::getExpressDate, request.getExpressDate());
        }
        rewardRedeemMapper.update(null, wrapper);
        return true;
    }

    @Override
    public Boolean verify(RewardRedeemVerifyReqVO request) {
        if (!checkVerifyParam(request)) {
            throw exception(ErrorCodeConstants.REWARD_REDEEM_VERIFY_NO_PARAM);
        }
        LambdaUpdateWrapper<RewardRedeemDO> wrapper = Wrappers.lambdaUpdate();
        wrapper.eq(RewardRedeemDO::getId, request.getId());
        wrapper.set(RewardRedeemDO::getStatus, RewardRedeemStatusEnum.REDEEMED.getValue());
        wrapper.set(StringUtils.isNotBlank(request.getRecipientName()), RewardRedeemDO::getRecipientName, request.getRecipientName());
        wrapper.set(StringUtils.isNotBlank(request.getRecipientPhoneNum()), RewardRedeemDO::getRecipientPhoneNum, request.getRecipientPhoneNum());
        wrapper.set(StringUtils.isNotBlank(request.getRecipientAddress()), RewardRedeemDO::getRecipientAddress, request.getRecipientAddress());
        wrapper.set(request.getCourierCompany() != null, RewardRedeemDO::getCourierCompany, request.getCourierCompany());
        wrapper.set(StringUtils.isNotBlank(request.getExpressNo()), RewardRedeemDO::getExpressNo, request.getExpressNo());
        wrapper.set(request.getExpressDate() != null, RewardRedeemDO::getExpressDate, request.getExpressDate());
        wrapper.set(StringUtils.isNotBlank(request.getExpressSender()), RewardRedeemDO::getExpressSender, request.getExpressSender());
        wrapper.set(RewardRedeemDO::getVerifyUser, request.getVerifyUser());
        wrapper.set(RewardRedeemDO::getVerifyTime, request.getVerifyTime());
        wrapper.set(RewardRedeemDO::getUpdater, SecurityFrameworkUtils.getLoginUserId());
        wrapper.set(RewardRedeemDO::getUpdateTime, new Date());
        rewardRedeemMapper.update(null, wrapper);
        return true;
    }

    @Override
    public Boolean batchVerify(RewardRedeemBatchVerifyReqVO request) {
        if (CollectionUtils.isEmpty(request.getIds())) {
            throw exception(ErrorCodeConstants.REWARD_REDEEM_NOT_EXIST);
        }
        List<String> errorIds = checkBatchVerifyParam(request);
        if (CollectionUtils.isNotEmpty(errorIds)) {
            throw exception(ErrorCodeConstants.REWARD_REDEEM_BATCH_VERIFY_ERROR, errorIds);
        }
        LambdaUpdateWrapper<RewardRedeemDO> updateWrapper = Wrappers.lambdaUpdate();
        updateWrapper.in(RewardRedeemDO::getId, request.getIds());
        updateWrapper.set(RewardRedeemDO::getStatus, RewardRedeemStatusEnum.REDEEMED.getValue());
        updateWrapper.set(RewardRedeemDO::getVerifyUser, request.getVerifyUser());
        updateWrapper.set(RewardRedeemDO::getVerifyTime, request.getVerifyTime());
        updateWrapper.set(RewardRedeemDO::getUpdater, SecurityFrameworkUtils.getLoginUserId());
        updateWrapper.set(RewardRedeemDO::getUpdateTime, new Date());
        this.update(null, updateWrapper);
        return true;
    }

    @Override
    public Boolean verifyBack(List<Long> redeemIds) {
        if (CollectionUtils.isEmpty(redeemIds)) {
            throw exception(ErrorCodeConstants.REWARD_REDEEM_NOT_EXIST);
        }
        LambdaUpdateWrapper<RewardRedeemDO> wrapper = Wrappers.lambdaUpdate();
        wrapper.in(RewardRedeemDO::getId, redeemIds);
        List<RewardRedeemDO> list = this.list(wrapper);
        if (CollectionUtils.isEmpty(list)) {
            throw exception(ErrorCodeConstants.REWARD_REDEEM_NOT_EXIST);
        }
        List<RewardRedeemDO> statusErrorList = list.stream().filter(item -> item.getStatus() != RewardRedeemStatusEnum.REDEEMED.getValue())
                .collect(Collectors.toList());
        if (CollectionUtils.isNotEmpty(statusErrorList)) {
            throw exception(ErrorCodeConstants.REWARD_REDEEM_VERIFY_BACK_STATUS);
        }
        LambdaUpdateWrapper<RewardRedeemDO> updateWrapper = Wrappers.lambdaUpdate();
        updateWrapper.in(RewardRedeemDO::getId, redeemIds);
        updateWrapper.set(RewardRedeemDO::getStatus, RewardRedeemStatusEnum.REDEEMING.getValue());
        updateWrapper.set(RewardRedeemDO::getVerifyUser, null);
        updateWrapper.set(RewardRedeemDO::getVerifyTime, null);
        updateWrapper.set(RewardRedeemDO::getUpdater, SecurityFrameworkUtils.getLoginUserId());
        updateWrapper.set(RewardRedeemDO::getUpdateTime, new Date());
        this.update(null, updateWrapper);
        return true;
    }

    @Override
    public Boolean export(RewardRedeemPageReqVO reqVO) {
        FileMakeReqDTO fileMakeReqDTO = new FileMakeReqDTO();
        fileMakeReqDTO.setType(REWARD_REDEEM_RECORD.getType());
        fileMakeReqDTO.setName("礼品兑换记录导出excel");
        fileMakeReqDTO.setFileSuffix("xlsx");
        fileMakeReqDTO.setTemporaryFile(true);
        fileMakeReqDTO.setUserType(2);
        fileMakeReqDTO.setLang(I18nMessage.getLang());
        fileMakeReqDTO.setRequestParams(JSONObject.toJSONString(reqVO));
        fileMakeApi.sendFileMake(fileMakeReqDTO);
        return true;
    }

    @Override
    public List<RewardRedeemPageRespVO> exportList(RewardRedeemPageReqVO request) {
        return rewardRedeemMapper.exportList(request);
    }

    @Override
    public Integer exportCount(RewardRedeemPageReqVO request) {
        return rewardRedeemMapper.pageCount(request);
    }

    @Override
    public RecordInfoImportRespVO recordImport(List<RedeemInfoImportExcelVO> dataList) {
        if (CollectionUtils.isEmpty(dataList)) {
            throw exception(ErrorCodeConstants.REDEEM_IMPORT_NO_RECORD);
        }
        if (dataList.size() > MAX_IMPORT_RECORD_COUNT) {
            throw exception(ErrorCodeConstants.REDEEM_IMPORT_MAX_COUNT, MAX_IMPORT_RECORD_COUNT);
        }
        Collection<CurrencyRespDTO> allCurrency = currencyApi.getAllCurrency().values();
        Map<String, CurrencyRespDTO> titleZhCurrencyMap = allCurrency.stream().collect(Collectors.toMap(CurrencyRespDTO::getTitleZh, Function.identity(), (c1, c2) -> c1));
        Map<String, CurrencyRespDTO> titleEnCurrencyMap = allCurrency.stream().collect(Collectors.toMap(CurrencyRespDTO::getTitleEn, Function.identity(), (c1, c2) -> c1));
        Map<String, ExpressRespDTO> nameExpressMap = expressApi.getAllExpress().stream().collect(Collectors.toMap(ExpressRespDTO::getCompanyName, Function.identity(), (c1, c2) -> c1));
        Map<String, String> failedInfo = validate(dataList, titleZhCurrencyMap, titleEnCurrencyMap, nameExpressMap);
        if (MapUtil.isNotEmpty(failedInfo)) {
            return RecordInfoImportRespVO.builder().redeemIdFailedMap(failedInfo).build();
        }

        List<RewardRedeemDO> saveDataList = dataList.stream().map(data -> {
            RewardRedeemDO rewardRedeemDO = new RewardRedeemDO();
            rewardRedeemDO.setId(Long.parseLong(data.getId()));
            DictDataRespDTO redeemTypeDict = DictFrameworkUtils.parseDictDataFromCacheWithMultiLang("way_of_receiving", data.getRedeemType());
            rewardRedeemDO.setRedeemType(Integer.parseInt(redeemTypeDict.getValue()));
            rewardRedeemDO.setExpenses(data.getExpenses());
            CurrencyRespDTO currencyRespDTO = titleZhCurrencyMap.get(data.getCurrency());
            if (currencyRespDTO == null) {
                currencyRespDTO = titleEnCurrencyMap.get(data.getCurrency());
            }
            rewardRedeemDO.setCurrency(Optional.ofNullable(currencyRespDTO).map(CurrencyRespDTO::getId).orElse(null));
            rewardRedeemDO.setCourierCompany(nameExpressMap.get(data.getCourierCompany()).getId());
            rewardRedeemDO.setExpressNo(data.getExpressNo());
            rewardRedeemDO.setExpressDate(DateUtil.parse(data.getExpressDate(), NORM_DATETIME_PATTERN));
            rewardRedeemDO.setExpressSender(data.getExpressSender());
            rewardRedeemDO.setRemark(data.getRemark());
            return rewardRedeemDO;
        }).collect(Collectors.toList());
        this.updateBatchById(saveDataList);
        return RecordInfoImportRespVO.builder().redeemIdFailedMap(Collections.emptyMap()).build();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean cancel(RewardRedeemIdReqVO req) {
        //取消兑换 状态改为已取消 回退扣减的积分 ，如果存在已过期的积分，则直接过期
        if (req.getId() == null) {
            throw exception(ErrorCodeConstants.REWARD_REDEEM_NOT_EXIST);
        }
        RewardRedeemDO rewardRedeemDO = this.getById(req.getId());
        if (rewardRedeemDO == null) {
            throw exception(ErrorCodeConstants.REWARD_REDEEM_NOT_EXIST);
        }
        if (rewardRedeemDO.getStatus() != RewardRedeemStatusEnum.REDEEMING.getValue()) {
            throw exception(ErrorCodeConstants.REDEEM_CANCEL_STATUS_ERROR);
        }
        LambdaUpdateWrapper<RewardRedeemDO> updateWrapper = Wrappers.lambdaUpdate();
        updateWrapper.eq(RewardRedeemDO::getId, req.getId());
        updateWrapper.eq(RewardRedeemDO::getStatus, RewardRedeemStatusEnum.REDEEMING.getValue());
        updateWrapper.set(RewardRedeemDO::getStatus, RewardRedeemStatusEnum.CANCELED.getValue());
        boolean updateSuccess = this.update(null, updateWrapper);
        if (!updateSuccess) {
            throw exception(ErrorCodeConstants.REDEEM_CANCEL_STATUS_ERROR);
        }
        Map<String, Object> extParam = new HashMap<>();
        extParam.put("redeemId", req.getId());
        extParam.put("redemptionNumber", rewardRedeemDO.getRedemptionNumber());
        memberUserScoreApi.operateScore(MemberUserScoreOperateReqDTO.builder()
                .memberId(rewardRedeemDO.getMemberId())
                .sourceType(ScoreSourceTypeEnum.EXCHANGE_REWARD_CANCEL)
                .scoreCount(rewardRedeemDO.getScoreCount())
                .extParam(extParam)
                .releationId(String.valueOf(req.getId()))
                .uniqueId(String.valueOf(req.getId()))
                .build());
        return true;
    }

    @Override
    public RedeemCancelCheckRespVO cancelCheck(RewardRedeemIdReqVO req) {
        RewardRedeemDO rewardRedeemDO = this.getById(req.getId());
        if (rewardRedeemDO == null) {
            throw exception(ErrorCodeConstants.REWARD_REDEEM_NOT_EXIST);
        }
        if (rewardRedeemDO.getStatus() != RewardRedeemStatusEnum.REDEEMING.getValue()) {
            throw exception(ErrorCodeConstants.REDEEM_CANCEL_STATUS_ERROR);
        }
        List<ReleationScoreExpireInfoDTO> scoreExpireInfo = memberUserScoreApi
                .getScoreExpireInfo(String.valueOf(req.getId()), ScoreSourceTypeEnum.EXCHANGE_REWARD);
        int totalExpireScore = scoreExpireInfo.stream().filter(e -> e.getExpireDate() != null && e.getExpireDate().compareTo(new Date()) < 0)
                .mapToInt(ReleationScoreExpireInfoDTO::getScoreCount).sum();
        return RedeemCancelCheckRespVO.builder().expireCount(totalExpireScore).build();
    }

    @Override
    public List<AppRewardRedeemListRespVO> recordList(Long memberId) {
        List<RewardRedeemDO> rewardRedeemDOList = rewardRedeemMapper.selectList("member_id", memberId);
        // 获得兑换记录列表，并根据时间倒序
        return rewardRedeemDOList.stream().map(rewardRedeemDO -> {
            RewardDO rewardDO = rewardService.getById(rewardRedeemDO.getRewardId());
            String status = rewardRedeemDO.getStatus().toString();
            return AppRewardRedeemListRespVO.builder()
                    .id(rewardRedeemDO.getId())
                    .redemptionNumber(rewardRedeemDO.getRedemptionNumber())
                    .createTime(rewardRedeemDO.getCreateTime())
                    .rewardCode(rewardRedeemDO.getRewardId())
                    .rewardTitleEn(rewardDO.getTitleEn())
                    .rewardTitleZh(rewardDO.getTitleZh())
                    .imgZh(rewardDO.getImgZh())
                    .imgEn(rewardDO.getImgEn())
                    .rewardCount(rewardRedeemDO.getRewardCount())
                    .scoreCount(rewardRedeemDO.getScoreCount())
                    .status(rewardRedeemDO.getStatus())
                    .statusEn(DictFrameworkUtils.getDictDataFromCache("reward_redeem_status", status).getLabelEn())
                    .statusZh(DictFrameworkUtils.getDictDataFromCache("reward_redeem_status", status).getLabel())
                    .build();
        }).sorted(Comparator.comparing(AppRewardRedeemListRespVO::getCreateTime).reversed()).collect(Collectors.toList());
    }

    private Map<String, String> validate(List<RedeemInfoImportExcelVO> dataList, Map<String, CurrencyRespDTO> titleZhCurrencyMap,
                                         Map<String, CurrencyRespDTO> titleEnCurrencyMap,
                                         Map<String, ExpressRespDTO> nameExpressMap) {
        LambdaQueryWrapper<RewardRedeemDO> wrapper = Wrappers.lambdaQuery();
        wrapper.in(RewardRedeemDO::getId, dataList.stream().map(RedeemInfoImportExcelVO::getId).collect(Collectors.toSet()));
        Map<Long, RewardRedeemDO> idRewardRedeemDOMap = rewardRedeemMapper.selectList(wrapper).stream()
                .collect(Collectors.toMap(RewardRedeemDO::getId, Function.identity(), (c1, c2) -> c1));
        Map<String, String> failedInfoMap = new HashMap<>();
        dataList.forEach(data -> {
            long id;
            try {
                id = Long.parseLong(data.getId());
            } catch (Exception e) {
                failedInfoMap.put(data.getId() == null ? "" : data.getId(), I18nMessage.getMessage("reward.redeem.not.exist"));
                return;
            }
            try {
                DateUtil.parse(data.getExpressDate(), NORM_DATETIME_PATTERN);
            } catch (Exception e) {
                failedInfoMap.put(data.getId() == null ? "" : data.getId(), I18nMessage.getMessage("date.format.error"));
                return;
            }
            RewardRedeemDO rewardRedeemDO = idRewardRedeemDOMap.get(id);
            if (rewardRedeemDO == null) {
                failedInfoMap.put(data.getId() == null ? "" : data.getId(), I18nMessage.getMessage("reward.redeem.not.exist"));
                return;
            }
            if (rewardRedeemDO.getStatus() != RewardRedeemStatusEnum.REDEEMING.getValue()) {
                failedInfoMap.put(data.getId(), I18nMessage.getMessage("reward.redeem.status.error"));
                return;
            }
            DictDataRespDTO redeemTypeDict = DictFrameworkUtils.parseDictDataFromCacheWithMultiLang("way_of_receiving", data.getRedeemType());
            if (redeemTypeDict == null) {
                failedInfoMap.put(data.getId(), I18nMessage.getMessage("dict.unknown.error", "way_of_receiving", data.getRedeemType()));
                return;
            }
            ExpressRespDTO expressRespDTO = nameExpressMap.get(data.getCourierCompany());
            if (expressRespDTO == null) {
                failedInfoMap.put(data.getId(), I18nMessage.getMessage("express.not.exist"));
                return;
            }
            CurrencyRespDTO currencyRespDTO = titleZhCurrencyMap.get(data.getCurrency());
            if (currencyRespDTO == null) {
                currencyRespDTO = titleEnCurrencyMap.get(data.getCurrency());
            }
            if (currencyRespDTO == null && StringUtils.isNotBlank(data.getCurrency())) {
                failedInfoMap.put(data.getId(), I18nMessage.getMessage("currency.not.exist"));
                return;
            }
            if (!canVerify(RewardRedeemVerifyDTO.builder()
                    .id(Long.parseLong(data.getId()))
                    .redeemType(Integer.parseInt(redeemTypeDict.getValue()))
                    .status(rewardRedeemDO.getStatus())
                    .expense(data.getExpenses())
                    .currency(Optional.ofNullable(currencyRespDTO).map(CurrencyRespDTO::getId).orElse(null))
                    .courierCompany(expressRespDTO.getId())
                    .expressSender(data.getExpressSender())
                    .expressDate(DateUtil.parse(data.getExpressDate(), NORM_DATETIME_PATTERN))
                    .expressNo(data.getExpressNo())
                    .recipientAddress(rewardRedeemDO.getRecipientAddress()).build())) {
                failedInfoMap.put(String.valueOf(rewardRedeemDO.getId()), I18nMessage.getMessage("reward.redeem.verify.no.param"));
            }
        });
        return failedInfoMap;
    }

    private List<String> checkBatchVerifyParam(RewardRedeemBatchVerifyReqVO request) {
        LambdaUpdateWrapper<RewardRedeemDO> wrapper = Wrappers.lambdaUpdate();
        wrapper.in(RewardRedeemDO::getId, request.getIds());
        Map<String, RewardRedeemDO> idRewardRedeemMap = this.list(wrapper).stream()
                .collect(Collectors.toMap(RewardRedeemDO::getRedemptionNumber, Function.identity(), (c1, c2) -> c1));
        return idRewardRedeemMap.keySet().stream().filter(coder -> {
            RewardRedeemDO rewardRedeemDO = idRewardRedeemMap.get(coder);
            if (rewardRedeemDO == null) {
                return true;
            }
            return !canVerify(RewardRedeemVerifyDTO.builder()
                    .id(rewardRedeemDO.getId())
                    .status(rewardRedeemDO.getStatus())
                    .redeemType(rewardRedeemDO.getRedeemType())
                    .expense(rewardRedeemDO.getExpenses())
                    .currency(rewardRedeemDO.getCurrency())
                    .courierCompany(rewardRedeemDO.getCourierCompany())
                    .expressDate(rewardRedeemDO.getExpressDate())
                    .recipientAddress(rewardRedeemDO.getRecipientAddress())
                    .expressNo(rewardRedeemDO.getExpressNo())
                    .expressSender(rewardRedeemDO.getExpressSender())
                    .build());
        }).collect(Collectors.toList());
    }

    private boolean checkVerifyParam(RewardRedeemVerifyReqVO request) {
        RewardRedeemDO rewardRedeemDO = getById(request.getId());
        if (rewardRedeemDO == null) {
            throw exception(ErrorCodeConstants.REWARD_REDEEM_NOT_EXIST);
        }
        return canVerify(RewardRedeemVerifyDTO.builder()
                .id(rewardRedeemDO.getId())
                .status(rewardRedeemDO.getStatus())
                .redeemType(rewardRedeemDO.getRedeemType())
                .expense(rewardRedeemDO.getExpenses())
                .currency(rewardRedeemDO.getCurrency())
                .courierCompany(request.getCourierCompany())
                .expressDate(request.getExpressDate())
                .recipientAddress(request.getRecipientAddress())
                .expressNo(request.getExpressNo())
                .expressSender(request.getExpressSender())
                .build());
    }

    private boolean canVerify(RewardRedeemVerifyDTO rewardRedeemVerifyDTO) {
        if (rewardRedeemVerifyDTO.getStatus() != RewardRedeemStatusEnum.REDEEMING.getValue()) {
            return false;
        }
        if (rewardRedeemVerifyDTO.getRedeemType() != RewardPickMethedEnum.SELF_PICKUP.getValue()) {
            if (rewardRedeemVerifyDTO.getExpense() == null) {
                return false;
            }
            if (rewardRedeemVerifyDTO.getCurrency() == null) {
                return false;
            }
            if (rewardRedeemVerifyDTO.getCourierCompany() == null) {
                return false;
            }
            if (StringUtils.isBlank(rewardRedeemVerifyDTO.getExpressNo())) {
                return false;
            }
            if (rewardRedeemVerifyDTO.getExpressDate() == null) {
                return false;
            }
            if (StringUtils.isBlank(rewardRedeemVerifyDTO.getExpressSender())) {
                return false;
            }
            if (StringUtils.isBlank(rewardRedeemVerifyDTO.getRecipientAddress())) {
                return false;
            }
        }
        return true;
    }

    private Long updateValidateParam(RewardRedeemUpdateReqVO request) {
        RewardRedeemDO rewardRedeemDO = getById(request.getId());
        if (rewardRedeemDO == null) {
            throw exception(ErrorCodeConstants.REWARD_REDEEM_NOT_EXIST);
        }
        if (rewardRedeemDO.getStatus() != RewardRedeemStatusEnum.REDEEMING.getValue()) {
            throw exception(ErrorCodeConstants.REWARD_REDEEM_STATUS_ERROR);
        }
        return rewardRedeemDO.getRewardId();
    }
}
